<?php

namespace Drupal\views_regex_functions\Plugin\views\field;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Field handler to flag the node type.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("views_regex_functions_field")
 */
class ViewsRegexFunctionsField extends FieldPluginBase implements ContainerFactoryPluginInterface {
  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RendererInterface $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('renderer')
    );

  }

  /**
   * {@inheritDoc}
   */
  public function query() {
    // Leave empty to avoid a query on this field.
    $this->field_alias = 'views_regex_functions_' . $this->position;
  }

    /**
     * @return mixed
     * Used to update the Views UI
     */
    public function adminSummary() {
        return $this->t(' @pattern on @field ', [
            '@pattern' => trim($this->options['pattern']),
            '@field' => trim($this->options['search_subject']),
        ]);
    }

  /**
   * Define the available options.
   *
   * @return string[]
   *   Returns the available options.
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['label']['default'] = NULL;

    $options['pattern'] = ['default' => ''];
    $options['search_subject'] = ['default' => ''];
    $options['replacement'] = ['default' => ''];
    $options['strip_tags'] = ['default' => 0];
    $options['allowed_tags'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['views_regex_functions.settings'];
  }

  /**
   * Provide the options form.
   *
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    $form['relationship']['#access'] = FALSE;

    $form['pattern'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Pattern: <span style="font-weight: normal;">Replacement Pattern to Search For</span>&nbsp;&nbsp;<span style="color: red;">*</span>'),
      '#description' => $this->t('Pattern must be a Regular Expression (Regex).'),
      '#default_value' => $this->options['pattern'],
    ];
    $form['search_subject'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Search Subject: <span style="font-weight: normal;">Views Field (Column) this replacement pattern should work on</span>&nbsp;&nbsp;<span style="color: red;">*</span>'),
      '#description' => $this->t('Input a value to compare the field against. Replacement variables may be used.'),
      '#default_value' => $this->options['search_subject'],
    ];
    $form['replacement'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Replacement: <span style="font-weight: normal;">Replacement Text or Regex Pattern to Replace With</span>'),
      '#default_value' => $this->options['replacement'],
    ];
    $form['replacements']['notice'] = [
      '#markup' => 'You may use any of these replacement variables in the Search Subject.',
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    ];
    $items = [];
    $views_fields = $this->view->display_handler->getHandlers('field');
    foreach ($views_fields as $field => $handler) {
      // We only use fields up to (not including) this one.
      if ($field == $this->options['id']) {
        break;
      }
      $items[] = "{{ $field }}";
    }
    $form['replacements']['variables'] = [
      '#theme' => 'item_list',
      '#items' => $items,
    ];
    $form['strip_tags'] = [
      '#prefix' => '<br><hr>',
      '#type' => 'checkbox',
      '#title' => $this->t('Strip HTML Tags from the Search Subject.'),
      '#default_value' => $this->options['strip_tags'],
    ];
    $form['allowed_tags'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed Tags'),
      '#description' => $this->t('Must be HTML tags <b><u>ONLY</u></b>. If empty, all tags will be removed. (Blank Spaces, Tabs or Line Breaks Removed Automatically from this Input.)'),
      '#default_value' => $this->options['allowed_tags'],
    ];
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    if (empty($values['options']['pattern']) || empty($values['options']['search_subject'])) {
      // Make sure that both the pattern and subject are not empty.
      if (empty($values['options']['pattern'])) {
        $form_state->setErrorByName('pattern', $this->t("Please specify a valid Regex formula in the pattern."));
      }
      if (empty($values['options']['search_subject'])) {
        $form_state->setErrorByName('search_subject', $this->t("A field must be specified in the search subject."));
      }
    }
    else {
      if (@preg_match($values['options']['pattern'], '') === FALSE) {
        $form_state->setErrorByName('pattern', $this->t("Please specify a valid Regex formula in the pattern."));
      }
    }
    if (!empty($values['options']['allowed_tags'])) {
      $values['options']['allowed_tags'] = preg_replace('/\s+/', '', $values['options']['allowed_tags']);
      if (!empty(strip_tags($values['options']['allowed_tags']))) {
        $form_state->setErrorByName('allowed tags', $this->t("Only HTML Tags are allowed in the Allowed Tags field."));
      }
    }
  }

  /**
   * Cleans a variable for handling later.
   *
   * @param object $var
   *   The string to be cleaned of html.
   *
   * @return string
   *   The trimmed and cleaned string.
   */
  public function cleanVar($var): string {
    $unparsed = $var->last_render ?? '';
    $allowed_tags = preg_replace('/\s+/', '', $this->options['allowed_tags']);
    if (empty($allowed_tags)) {
      $allowed_tags = '';
    }
    return $this->options['strip_tags'] ? trim(strip_tags($unparsed, $allowed_tags)) : trim($unparsed);
  }

  /**
   * {@inheritDoc}
   */
  public function render(ResultRow $values) {
    $pattern = $this->options['pattern'];
    $search_subject = $this->options['search_subject'];
    $replacement = $this->options['replacement'];

    if (empty($replacement)) {
      $replacement = '';
    }

    // Gather field information.
    $fields = $this->view->display_handler->getHandlers('field');
    $labels = $this->view->display_handler->getFieldLabels();
    // Search through field information for possible replacement variables.
    foreach ($labels as $key => $var) {
      // If we find a replacement variable, replace it.
      if (strpos($search_subject, "{{ $key }}") !== FALSE) {
        $field = $this->cleanVar($fields[$key]);
        $search_subject = str_replace("{{ $key }}", $field, $search_subject);
      }
    }

    return preg_replace($pattern, $replacement, $search_subject);
  }

  /**
   * {@inheritDoc}
   */
  public function clickSortable() {
    return FALSE;
  }

}
