<?php

namespace Drupal\views_regex_functions\Plugin\views\filter;

use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\FieldAPIHandlerTrait;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\views\ResultRow;
use Drupal\Core\Url;
use Drupal\views\ViewExecutable;
use Drupal\views\Plugin\views\cache\CachePluginBase;

/**
 * Filter content by other view results set.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("views_regex_functions_filter")
 */
class ViewsRegexFunctionsFilter extends FilterPluginBase {
    /**
     * Cannot be exposed.
     */
    public function canExpose() {
        return FALSE;
    }

    /**
     * @return mixed
     * Used to update the Views UI
     */
    public function adminSummary() {
        return $this->t(' @pattern on @field ', [
            '@pattern' => trim($this->options['pattern']),
            '@field' => trim($this->options['search_subject']),
        ]);
    }

    /**
     * {@inheritdoc}
     */
    protected function defineOptions() {
        $options = parent::defineOptions();
        $options['remove_rows'] = ['default' => 0];
        $options['replacement'] = ['default' => ''];
        $options['pattern'] = ['default' => ''];
        $options['search_subject'] = ['default' => ''];
        $options['strip_tags'] = ['default' => 0];
        $options['allowed_tags'] = ['default' => ''];
        return $options;
    }

    /**
     * {@inheritdoc}
     */
    public function buildOptionsForm(&$form, FormStateInterface $form_state) {
        parent::buildOptionsForm($form, $form_state);

    }

    /**
     * {@inheritdoc}
     */
    public function showValueForm(&$form, FormStateInterface $form_state): void {
        parent::showValueForm($form, $form_state);
        $form['remove_rows'] = [
            '#prefix' => '<hr>',
            '#type' => 'checkbox',
            '#title' => $this->t('Remove Rows with Pattern'),
            '#description' => $this->t('If unchecked, only rows with the pattern in the search subject will show.'),
            '#default_value' => $this->options['remove_rows'],
        ];
        $form['pattern'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Pattern: <span style="font-weight: normal;">Pattern to Search For</span>&nbsp;&nbsp;<span style="color: red;">*</span>'),
            '#description' => $this->t('Pattern must be a Regular Expression (Regex).'),
            '#default_value' => $this->options['pattern'],
        ];
        $form['search_subject'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Search Subject: <span style="font-weight: normal;">Views Field (Column) this pattern should work on</span>&nbsp;&nbsp;<span style="color: red;">*</span>'),
            '#description' => $this->t('USE ONLY ONE OF THE REPLACEMENT VARIABLES BELOW.'),
            '#default_value' => $this->options['search_subject'],
        ];
        $form['replacements']['notice'] = [
            '#markup' => 'You may use any of these replacement variables in the Search Subject.',
            '#prefix' => '<p>',
            '#suffix' => '</p>',
        ];
        $items = [];
        $views_fields = $this->view->display_handler->getHandlers('field');
        foreach ($views_fields as $field => $handler) {
            $items[] = "{{ $field }}";
        }
        $form['replacements']['variables'] = [
            '#theme' => 'item_list',
            '#items' => $items,
        ];
        $form['strip_tags'] = [
            '#prefix' => '<br><hr>',
            '#type' => 'checkbox',
            '#title' => $this->t('Strip HTML Tags from the Search Subject.'),
            '#default_value' => $this->options['strip_tags'],
        ];
        $form['allowed_tags'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Allowed Tags'),
            '#description' => $this->t('Must be HTML tags <b><u>ONLY</u></b>. If empty, all tags will be removed. (Blank Spaces, Tabs or Line Breaks Removed Automatically from this Input.)'),
            '#default_value' => $this->options['allowed_tags'],
        ];
    }

    public function validateOptionsForm(&$form, FormStateInterface $form_state) {
        $values = $form_state->getValues();

        if (empty($values['options']['pattern']) || empty($values['options']['search_subject'])) {
            // Make sure that both the pattern and subject are not empty.
            if (empty($values['options']['pattern'])) {
                $form_state->setErrorByName('pattern', $this->t("Please specify a valid Regex formula in the pattern."));
            }
            if (empty($values['options']['search_subject'])) {
                $form_state->setErrorByName('search_subject', $this->t("A field must be specified in the search subject."));
            }
        }
        else {
            if (@preg_match($values['options']['pattern'], '') === FALSE) {
                $form_state->setErrorByName('pattern', $this->t("Please specify a valid Regex formula in the pattern."));
            }
            if(substr(trim($values['options']['search_subject']), 0, 2) != "{{" || count(explode('{{', $values['options']['search_subject'])) != 2 ||
                substr(trim($values['options']['search_subject']), -2) != "}}" || count(explode('}}', $values['options']['search_subject'])) != 2) {
                $form_state->setErrorByName('search_subject', $this->t("Only the replacement field is allowed in this box. No other text allowed. Please review the Search Subject to ensure it is correct."));
            }
        }
        if (!empty($values['options']['allowed_tags'])) {
            $values['options']['allowed_tags'] = preg_replace('/\s+/', '', $values['options']['allowed_tags']);
            if (!empty(strip_tags($values['options']['allowed_tags']))) {
                $form_state->setErrorByName('allowed tags', $this->t("Only HTML Tags are allowed in the Allowed Tags field."));
            }
        }
    }

    /**
     * Called to add the filter to a query.
     *
     * Since we do everything post execute, we need this function here so the view does not error out by trying to add information into the query itself.
     */
    public function query() {}
}